import requests, json
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from zeep import Client


class ParsianWebcoIr:
    """
        token
        TemplateID
        MessageVars
        Receiver
        delay
    """
    TOKEN = None
    HEADERS = {"Content-Type": "application/x-www-form-urlencoded"}
    def __init__(self, mobile, *args, **kwargs):
        try:
            # Give TOKEN from DJANGO_IRAN_SMS { PARSIAN_WEBCO_IR { TOKEN } }
            if not hasattr(settings, 'DJANGO_IRAN_SMS'):
                raise ImproperlyConfigured('DJANGO_IRAN_SMS must be defined in settings.py .')

            if 'PARSIAN_WEBCO_IR' not in settings.DJANGO_IRAN_SMS:
                raise ImproperlyConfigured('PARSIAN_WEBCO_IR must be defined in settings.py -> DJANGO_IRAN_SMS.')

            if 'API_KEY' not in settings.DJANGO_IRAN_SMS['PARSIAN_WEBCO_IR']:
                raise ImproperlyConfigured('API_KEY must be defined in settings.py -> DJANGO_IRAN_SMS:PARSIAN_WEBCO_IR.')
        except ImproperlyConfigured as e:
            print(f"Configuration Error: {e}")
            raise
        else:
            self.TOKEN = settings.DJANGO_IRAN_SMS['PARSIAN_WEBCO_IR']['API_KEY']
        self.RECEIVER = mobile

    def send_otp_code(self, code, template_id=None):
        try:
            if not template_id:
                if not hasattr(settings, 'DJANGO_IRAN_SMS'):
                    raise ImproperlyConfigured('DJANGO_IRAN_SMS must be defined in settings.py .')

                if 'PARSIAN_WEBCO_IR' not in settings.DJANGO_IRAN_SMS:
                    raise ImproperlyConfigured('PARSIAN_WEBCO_IR must be defined in settings.py -> DJANGO_IRAN_SMS.')

                if 'TEMPLATES' not in settings.DJANGO_IRAN_SMS['PARSIAN_WEBCO_IR']:
                    raise ImproperlyConfigured('TEMPLATES must be defined in settings.py -> DJANGO_IRAN_SMS:PARSIAN_WEBCO_IR.')

                if 'OTP_CODE' not in settings.DJANGO_IRAN_SMS['PARSIAN_WEBCO_IR']['TEMPLATES']:
                    raise ImproperlyConfigured('OTP_CODE must be defined in settings.py -> DJANGO_IRAN_SMS:PARSIAN_WEBCO_IR:TEMPLATES.')

                template_id = settings.DJANGO_IRAN_SMS['PARSIAN_WEBCO_IR']['TEMPLATES']['OTP_CODE']

            api_url = 'https://api.parsianwebco.ir/webservice-send-sms/send'
            data = {
                'token': self.TOKEN,
                'TemplateID': template_id,
                'MessageVars': code,
                'Receiver': self.RECEIVER,
                'delay': 1
            }
            return json.loads(requests.post(url=api_url, data=data, headers=self.HEADERS).content)
            """
                response:
                    status:
                        200 ok
                        100 faild
                        401 no authenticated
            """
        except ImproperlyConfigured as e:
            print(f"Configuration Error: {e}")
            raise
        return False

    def send_message(self, message, template_id):
        try:
            api_url = 'https://api.parsianwebco.ir/webservice-send-sms/send'
            data = {
                'token': self.TOKEN,
                'TemplateID': template_id,
                'MessageVars': message,
                'Receiver': self.RECEIVER,
                'delay': 1
            }
            return json.loads(requests.post(url=api_url, data=data, headers=self.HEADERS).content)
            """
                response:
                    status:
                        200 ok
                        100 faild
                        401 no authenticated
            """
        except:
            return False


class MeliPayamakCom:
    '''
    username
    password
    from
    to
    text
    '''
    USERNAME = None
    PASSWORD = None
    FROM = None

    def __init__(self, mobile, *args, **kwargs):
        try:
            # Give TOKEN from DJANGO_IRAN_SMS { PARSIAN_WEBCO_IR { TOKEN } }
            if not hasattr(settings, 'DJANGO_IRAN_SMS'):
                raise ImproperlyConfigured('DJANGO_IRAN_SMS must be defined in settings.py .')

            if 'MELI_PAYAMAK_COM' not in settings.DJANGO_IRAN_SMS:
                raise ImproperlyConfigured('MELI_PAYAMAK_COM must be defined in settings.py -> DJANGO_IRAN_SMS.')

            if 'USERNAME' not in settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']:
                raise ImproperlyConfigured('USERNAME must be defined in settings.py -> DJANGO_IRAN_SMS:MELI_PAYAMAK_COM.')

            if 'PASSWORD' not in settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']:
                raise ImproperlyConfigured('PASSWORD must be defined in settings.py -> DJANGO_IRAN_SMS:MELI_PAYAMAK_COM.')
            
            if 'FROM' not in settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']:
                raise ImproperlyConfigured('FROM must be defined in settings.py -> DJANGO_IRAN_SMS:MELI_PAYAMAK_COM.')
        
        except ImproperlyConfigured as e:
            print(f"Configuration Error: {e}")
            raise
        else:
            self.USERNAME = settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']['USERNAME']
            self.PASSWORD = settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']['PASSWORD']
            self.FROM = settings.DJANGO_IRAN_SMS['MELI_PAYAMAK_COM']['FROM']
        self.RECEIVER = mobile

    def send_message(self, message):
        try:
            client = Client(wsdl='https://api.payamak-panel.com/post/Send.asmx?wsdl')
            data = {
                'username': self.USERNAME,
                'password': self.PASSWORD,
                'from': self.FROM,
                'to': self.RECEIVER,
                'text': message,
                'isflash': False
            }

            response = client.service.SendSimpleSMS2(**data)
            return response
            """
                response:
                    status:
                        recld (Unique value for each successful submission)
                        0   The username or password is incorrect.
                        2   Not enough credit.
                        3   Limit on daily sending.
                        4   Limit on sending volume.
                        5   The sender's number is not valid.
                        6   The system is being updated.
                        7   The text contains the filtered word.
                        9   Sending from public lines via web service is not possible.
                        10  The desired user is not active.
                        11  Not sent.
                        12  The user's credentials are not complete.
                        14  The text contains a link.
                        15  Sending to more than 1 mobile number is not possible without inserting "لغو11".
                        16  No recipient number found
                        17  The text of the SMS is empty.
                        35  In REST, it means that the number is on the blacklist of communications.
            """
        except:
            return False


class KavenegarCom:
    '''
    API_KEY
    receptor
    message
    sender
    '''
    API_KEY = None
    SENDER = None
    
    def __init__(self, mobile, *args, **kwargs):
        try:
            # Give TOKEN from DJANGO_IRAN_SMS { PARSIAN_WEBCO_IR { TOKEN } }
            if not hasattr(settings, 'DJANGO_IRAN_SMS'):
                raise ImproperlyConfigured('DJANGO_IRAN_SMS must be defined in settings.py .')

            if 'KAVENEGAR_COM' not in settings.DJANGO_IRAN_SMS:
                raise ImproperlyConfigured('KAVENEGAR_COM must be defined in settings.py -> DJANGO_IRAN_SMS.')

            if 'API_KEY' not in settings.DJANGO_IRAN_SMS['KAVENEGAR_COM']:
                raise ImproperlyConfigured('API_KEY must be defined in settings.py -> DJANGO_IRAN_SMS:KAVENEGAR_COM.')

            if 'FROM' not in settings.DJANGO_IRAN_SMS['KAVENEGAR_COM']:
                raise ImproperlyConfigured('FROM must be defined in settings.py -> DJANGO_IRAN_SMS:KAVENEGAR_COM.')

        except ImproperlyConfigured as e:
            print(f"Configuration Error: {e}")
            raise
        else:
            self.API_KEY = settings.DJANGO_IRAN_SMS['KAVENEGAR_COM']['API_KEY']
            self.SENDER = settings.DJANGO_IRAN_SMS['KAVENEGAR_COM']['FROM']
        self.RECEIVER = mobile


    def send_message(self, message):
        try:
            api_url = f'https://api.kavenegar.com/v1/{self.API_KEY}/sms/send.json'
            data = {
                'sender': self.SENDER,
                'receptor': self.RECEIVER,
                'message': message,
            }
            response = requests.post(url=api_url, data=data)
            return response
            """
                response:
                    messageid Unique identifier of this SMS (To know the status of the sent SMS, this value is the input to the Status method.)
                    status:
                        200   if status is 10 & 5 , message received.
                        400   The parameters are incomplete.
                        401   Account has been deactivated.
                        403   The API-Key identification code is not valid.
                        406   Empty mandatory parameters sent.
                        411   The recipient is invalid.
                        412   The sender is invalid.
                        413   The message is empty or the message length exceeds the allowed limit. The maximum length of the entire SMS text is 900 characters.
                        414   The request volume exceeds the allowed limit, sending SMS: maximum 200 records per call and status control: maximum 500 records per call
                        416   The originating service IP does not match the settings.
                        418   Your credit is not sufficient.
                        451   Excessive calls within a specific time period are restricted to IP addresses.
            """
        except:
            return False



import json
from unittest.mock import patch, MagicMock
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from django.contrib.auth import get_user_model
from datetime import datetime, timedelta
from .models import OTPCode
from .models import OTPCode, User as CustomUser


class TestParsianWebcoIrService(APITestCase):
    user = get_user_model()

    @patch('drfiransms.views.ParsianWebcoIr.send_otp_code')
    @patch('drfiransms.views.sms_service_check', return_value=True)
    @patch('drfiransms.views.init_check', return_value={'SMS_BACKEND': 'PARSIAN_WEBCO_IR'})
    def test_send_otp_code_success(self, mock_init, mock_check, mock_send):
        mock_send.return_value = {'status': 200}
        client = APIClient()
        response = client.post('/drf/send/otpcode/', {'mobile_number': '09167332792'})
        self.assertEqual(response.status_code, 200)
        mock_send.assert_called_once()
    

    @patch('drfiransms.views.ParsianWebcoIr.send_message')
    @patch('drfiransms.views.sms_service_check', return_value=True)
    @patch('drfiransms.views.init_check', return_value={'SMS_BACKEND': 'PARSIAN_WEBCO_IR'})
    def test_send_message_success(self, mock_init, mock_check, mock_send):
        mock_send.return_value = {'status': 200}
        client = APIClient()
        response = client.post('/drf/send/message/', {'mobile_number': '09167332792', 'message_text': 'hello louristan.', 'template_id': '1'})
        self.assertEqual(response.status_code, 200)
        mock_send.assert_called_once()

    
    def test_authentication_success(self):
        mobile_number = '09167332792'
        otp = OTPCode.objects.create(
            mobile= mobile_number,
        )
        otp_code = OTPCode.objects.filter(mobile= mobile_number).first().code
        client = APIClient()
        response = client.post('/drf/auth/', {'mobile_number': mobile_number, 'code': otp_code, 'group': '1'})

        self.assertEqual(response.status_code, 200)


class TestMelipayamakComService(APITestCase):

    @patch('drfiransms.views.MeliPayamakCom.send_message', return_value='123456')
    @patch('drfiransms.views.sms_service_check', return_value=True)
    @patch('drfiransms.views.init_check', return_value={'SMS_BACKEND': 'MELI_PAYAMAK_COM'})
    def test_send_otp_code_success(self, mock_init, mock_check, mock_send):
        client = APIClient()
        response = client.post('/drf/send/otpcode/', {'mobile_number': '09167332792'})
        self.assertEqual(response.status_code, 200)
        mock_send.assert_called_once()

    
    @patch('drfiransms.views.MeliPayamakCom.send_message', return_value='123456')
    @patch('drfiransms.views.sms_service_check', return_value=True)
    @patch('drfiransms.views.init_check', return_value={'SMS_BACKEND': 'MELI_PAYAMAK_COM'})
    def test_send_message_success(self, mock_init, mock_check, mock_send):
        client = APIClient()
        response = client.post('/drf/send/message/', {'mobile_number': '09167332792', 'message_text': 'hello louristan'})
        self.assertEqual(response.status_code, 200)
        mock_send.assert_called_once()

    
    def test_authentication_success(self):
        mobile_number = '09167332792'

        otp = OTPCode.objects.create(
            mobile= mobile_number
        )

        otp_code = OTPCode.objects.filter(mobile=mobile_number).first().code

        client = APIClient()
        response = client.post('/drf/auth/', {'mobile_number': mobile_number, 'code': otp_code, 'group': '0'})
        print(response)
        self.assertEqual(response.status_code, 200)




from django.apps import AppConfig


class DrfiransmsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'drfiransms'

    def ready(self):
        import drfiransms.signals



"""
DJANGO_IRAN_SMS = {
    'AUTHENTICATION': 'rest_framework_simplejwt',   # rest_framework_simplejwt
    'SMS_BACKEND': 'PARSIAN_WEBCO_IR',              # PARSIAN_WEBCO_IR , MELI_PAYAMAK_COM , KAVENEGAR_COM
    'OTP_CODE': {
        'LENGTH': 8,                                # DEFAULT 8
        'EXPIRE_PER_MINUTES': 4,                    # DEFAULT 4
    },
    'PARSIAN_WEBCO_IR': {
        'API_KEY': '',
        'TEMPLATES': {
            'OTP_CODE': 1,
        }
    },
    'MELI_PAYAMAK_COM': {
        'USERNAME': '',
        'PASSWORD': '',
        'FROM': '',
    },
    'KAVENEGAR_COM': {
        'API_KEY': '656F6635756C485658666F6A52307562456C4F5043714769597A58434D2B527974434534672B50445736553D',
        'FROM': '2000660110'
    }

    ...
}
"""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

SERVICE_NAME = 'DJANGO_IRAN_SMS'

AUTH_SERVICE = {
    'JWT_SIMPLE_DRF': 'rest_framework_simplejwt'
}

SMS_SERVICES = {
    'PARSIAN_WEBCO_IR': 'PARSIAN_WEBCO_IR',
    'MELI_PAYAMAK_COM': 'MELI_PAYAMAK_COM'
}

def init_check():
    try:
        if not hasattr(settings, SERVICE_NAME):
            raise ImproperlyConfigured(f'{SERVICE_NAME} must be defined in settings.py.')
        else:
            auth_service = getattr(settings, SERVICE_NAME).get('AUTHENTICATION')
            if not auth_service:
                raise ImproperlyConfigured(f'AUTHENTICATION kay must be defined in {SERVICE_NAME} for choice sms service provider.')
            sms_service = getattr(settings, SERVICE_NAME).get('SMS_BACKEND')
            if not sms_service:
                raise ImproperlyConfigured(f'SMS_BACKEND key must be defined in {SERVICE_NAME} for get login access to sms provider.')
        return {'AUTHENTICATION': auth_service, 'SMS_BACKEND': sms_service}
    except ImproperlyConfigured as e:
        print(f"Configuration Error: {e}")
        raise
    except:
        pass
    return False

def otp_code_length():
    try:
        otp_len = getattr(settings, SERVICE_NAME, {}).get('OTP_CODE', {}).get('LENGTH', 6)
        if not isinstance(otp_len, int):
            raise ImproperlyConfigured("OTP_CODE LENGTH must be int.")
        if otp_len < 3 or otp_len > 10:
            raise ImproperlyConfigured("OTP_CODE LENGTH must be less than or equal to 10 and greater than or equal to 3.")
        return otp_len
    except ImproperlyConfigured as e:
        print(f"Configuration Error: {e}")
        raise
    except:
        return 6

def otp_code_expire():
    try:
        otp_exp = getattr(settings, SERVICE_NAME, {}).get('OTP_CODE', {}).get('EXPIRE_PER_MINUTES', 2)
        if not isinstance(otp_exp, int):
            raise ImproperlyConfigured("OTP_CODE EXPIRE must be int.")
        if otp_exp <= 0:
            raise ImproperlyConfigured("OTP_CODE EXPIRE must be greater than 0.")
        return otp_exp
    except ImproperlyConfigured as e:
        print(f"Configuration Error: {e}")
        raise
    except:
        return 2


def sms_service_check(sms_service=None):
    try:
        if init_check():
            if sms_service not in settings.DJANGO_IRAN_SMS:
                raise ImproperlyConfigured(f'{sms_service} must be defiend in settings.py -> DJANGO_IRAN_SMS.')
            return True
    except ImproperlyConfigured as e:
        print(f"Configuration Error: {e}")
        raise
    except:
        pass
    return False





from django.contrib import admin
from .models import User, OTPCode

@admin.register(User)
class MobileAdmin(admin.ModelAdmin):
    list_display = ['id', 'user__id', 'user__username', 'mobile', 'user__is_active']


@admin.register(OTPCode)
class OTPCodeAdmin(admin.ModelAdmin):
    list_display = ['id', 'code', 'mobile', 'created_at']
    ordering = ('-created_at', )




from django.urls import path
from .views import OTPCodeSend ,Authentication, MessageSend

urlpatterns = [
    path('send/message/', MessageSend.as_view()),
    path('send/otpcode/', OTPCodeSend.as_view()),
    path('auth/', Authentication.as_view()),
]



from django.db import models
from django.contrib.auth.models import User as default_user
from django.utils.timezone import now, timedelta
from random import randint
from .settings import otp_code_length, otp_code_expire


class User(models.Model):
    user = models.OneToOneField(default_user, on_delete=models.CASCADE, related_name='mobile')
    mobile = models.CharField(max_length=11)
    group = models.IntegerField(default=0, help_text='choice group type or user level, with numbers.')

    def __str__(self):
        return f'{self.user.username} | {self.mobile}'


class OTPCode(models.Model):
    code = models.CharField(max_length=10)
    mobile = models.CharField(max_length=11)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.code} -> {self.mobile} | {self.created_at}'

    def save(self, *args, **kwargs):
        try:
            objs = OTPCode.objects.filter(mobile=self.mobile)
            if objs:
                if now().timestamp() > (objs.first().created_at + timedelta(seconds=otp_code_expire() * 60)).timestamp():
                    objs.delete()
                else:
                    return 409

            code = str(randint(int('1' + (otp_code_length() - 1) * '0'), int(otp_code_length() * '9')))
            self.code = code
            super().save(*args, **kwargs)
            return self
        except:
            pass
        return 500

    def check_code(self):
        try:
            if now().timestamp() <= (self.created_at + timedelta(seconds=otp_code_expire() * 60)).timestamp():
                self.delete()
                return True
            self.delete()
        except:
            pass
        return False




from rest_framework.views import APIView
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_204_NO_CONTENT, HTTP_500_INTERNAL_SERVER_ERROR, HTTP_409_CONFLICT, HTTP_502_BAD_GATEWAY, HTTP_401_UNAUTHORIZED
from .models import User
from .serializers import OTPCodeSerializer
from .validators import mobile_number as mobile_validator
from .services import ParsianWebcoIr, MeliPayamakCom, KavenegarCom
from .settings import otp_code_expire, init_check, sms_service_check


class OTPCodeSend(APIView):
    permission_classes = (AllowAny, )
    serializer_class = OTPCodeSerializer
    model = serializer_class.Meta.model

    def post(self, request):
        """
        prams:
            mobile_number:   str (len: 11)   (exp: 09211892425)

        response:
            HTTP_409_CONFLICT               {'error': 'Try again after - minutes..'}
            HTTP_204_NO_CONTENT             {'error': [params requirements and validations]}
            HTTP_500_INTERNAL_SERVER_ERROR  {'error': 'contact the support..'}
            HTTP_200_OK                     {'details': 'The OTP code was sent correctly.'}
            HTTP_502_BAD_GATEWAY            {'details': 'The SMS service provider was unable to process the request.'}
            HTTP_401_UNAUTHORIZED           {'details': 'Authentication is not accepted, check your token and if the token has not been received, obtain it from your SMS service provider.'}
        """
        try:
            assert 'mobile_number' in request.data, 'mobile_number is required.'
            mobile_number = request.data['mobile_number']
            assert mobile_number, 'mobile_number may not be blank.'
            mobile_number_isvalid = mobile_validator(mobile_number)
            assert mobile_number_isvalid == True, mobile_number_isvalid
            
            icheck = init_check()
            if icheck and isinstance(icheck, dict) and 'SMS_BACKEND' in icheck:
                sms_service = icheck['SMS_BACKEND']
                if sms_service_check(sms_service):
                    otp_code = self.model(mobile=mobile_number)
                    obj = otp_code.save()
                    if isinstance(obj, self.model):
                        # send sms:                    
                        match sms_service:
                            case 'PARSIAN_WEBCO_IR':
                                service = ParsianWebcoIr(mobile=obj.mobile)
                                response = service.send_otp_code(code=obj.code)
                                if isinstance(response, dict) and 'status' in response:
                                    if response['status'] == 200:
                                        return Response({'details': 'The OTP code was sent correctly.'}, status=HTTP_200_OK)
                                    elif response['status'] == 100:
                                        return Response({'details': 'The SMS service provider was unable to process the request.'}, status=HTTP_502_BAD_GATEWAY)
                                    elif response['status'] == 401:
                                        return Response({'details': 'Authentication is not accepted, check your token and if the token has not been received, obtain it from your SMS service provider.'}, status=HTTP_401_UNAUTHORIZED)
                    
                            case 'MELI_PAYAMAK_COM':
                                service = MeliPayamakCom(mobile=obj.mobile)
                                response = service.send_message(message=obj.code)
                                if response in [0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, 16, 17, 35]:
                                    return Response({'details': 'The SMS service provider was unable to process the request.', 'errorCode': response}, status=HTTP_502_BAD_GATEWAY)
                                else:
                                    return Response({'details': 'The OTP code was sent correctly.', 'smsID': response}, status=HTTP_200_OK)

                            case 'KAVENEGAR_COM':
                                service = KavenegarCom(mobile=mobile_number)
                                response = service.send_message(message=obj.code)
                                try:
                                    response_json = response.json()
                                    entries = response_json.get('entries', [])
                                    _return = response_json.get('return', [])
                                    if entries:
                                        response_data = entries[0]
                                        if response.status_code == 200 and response_data.get('status') in [5, 10]:
                                            return Response({'details': 'The OTP code was sent correctly.', 'messageid': response_data.get('messageid', '')}, status=HTTP_200_OK)
                                        else:
                                            return Response({'details': 'The SMS service provider was unable to process the request.', 'errorcode': response_data.get('status', ''), 'errortext': response_data.get('statustext', ''), 'message': response_data.get('message', '')}, HTTP_502_BAD_GATEWAY)
                                    elif  _return:
                                        return Response({'details': 'The SMS service provider was unable to process the request.', 'message': _return.get('message', '')}, status=HTTP_502_BAD_GATEWAY)
                                except (ValueError, KeyError, IndexError) as e:
                                    return Response({'details': 'Invalid response structure.', 'error': str(e)}, status=HTTP_502_BAD_GATEWAY)


                    else:
                        if obj == 409:
                            return Response({'error': f'Try again after {otp_code_expire()} minutes.'}, status=HTTP_409_CONFLICT)
        except AssertionError as e:
            return Response({'error': str(e)}, status=HTTP_204_NO_CONTENT)
        except:
            pass
        return Response({'error': 'An error occurred while generating or sending the otpcode, please contact the www.chelseru.com support team.'}, status=HTTP_500_INTERNAL_SERVER_ERROR)


class Authentication(APIView):
    permission_classes = (AllowAny, )
    serializer_class = OTPCodeSerializer
    model = serializer_class.Meta.model

    def post(self, request):
        """
        prams:
            mobile_number:   str (len: 11)     (exp: 09211892425)
            code: str (len: otp_code_length()) (exp: 652479)
            pre_username: str (len: 7)         (exp: service)

        response:
            HTTP_204_NO_CONTENT             {'error': [params requirements and validations]}
            HTTP_500_INTERNAL_SERVER_ERROR  {'error': 'contact the support..'}
            HTTP_200_OK                     {'access': '', 'refresh': ''}
        """
        try:
            assert 'mobile_number' in request.data, 'mobile_number is required.'
            mobile_number = request.data['mobile_number']
            assert mobile_number, 'mobile_number may not be blank.'
            mobile_number_isvalid = mobile_validator(mobile_number)
            assert mobile_number_isvalid == True, mobile_number_isvalid
            assert 'code' in request.data, 'code is required.'
            
            icheck = init_check()
            if icheck and isinstance(icheck, dict) and 'AUTHENTICATION' in icheck:
                otp_code = request.data['code']
                otp = self.model.objects.filter(mobile=mobile_number).filter(code=otp_code).first()
                if otp:
                    if otp.check_code():
                        # login / signup
                        group = int(request.data['group']) if 'group' in request.data else 0
                        user, created = User.objects.get_or_create(mobile=mobile_number, group=group)
                        if user:
                            auth_service = icheck['AUTHENTICATION']
                            match auth_service:
                                case 'rest_framework_simplejwt':
                                    from rest_framework_simplejwt.tokens import RefreshToken, AccessToken, BlacklistedToken
                                    access_token = AccessToken.for_user(user=user.user)
                                    refresh_token = RefreshToken.for_user(user=user.user)
                                    return Response({'access': str(access_token), 'refresh': str(refresh_token)}, status=HTTP_200_OK)
                        
                else:
                    return Response({'error': 'The code sent to this mobile number was not found.'}, status=HTTP_401_UNAUTHORIZED)
        except AssertionError as e:
            return Response({'error': str(e)}, status=HTTP_204_NO_CONTENT)
        except:
            pass
        return Response({'error': 'An error occurred while generating or sending the otpcode, please contact the www.chelseru.com support team.'}, status=HTTP_500_INTERNAL_SERVER_ERROR)


class MessageSend(APIView):
    permission_classes = (AllowAny, )
    #serializer_class = MessageSerializer
    #model = serializer_class.Meta.model

    def post(self, request):
        """
        prams:
            mobile_number:   str (len: 11)   (exp: 09211892425)
            message_text:    str (len: 290)

        response:
            HTTP_204_NO_CONTENT             {'error': [params requirements and validations]}
            HTTP_500_INTERNAL_SERVER_ERROR  {'error': 'contact the support..'}
            HTTP_200_OK                     {'details': 'The Message was sent correctly.'}
            HTTP_502_BAD_GATEWAY            {'details': 'The SMS service provider was unable to process the request.'}
            HTTP_401_UNAUTHORIZED           {'details': 'Authentication is not accepted, check your token and if the token has not been received, obtain it from your SMS service provider.'}
        """
        try:
            assert 'mobile_number' in request.data, 'mobile_number is required.'
            mobile_number = request.data['mobile_number']
            assert mobile_number, 'mobile_number may not be blank.'
            mobile_number_isvalid = mobile_validator(mobile_number)
            assert mobile_number_isvalid == True, mobile_number_isvalid
            
            assert 'message_text' in request.data, 'message_text is required.'
            message_text = request.data['message_text']

            icheck = init_check()
            if icheck and isinstance(icheck, dict) and 'SMS_BACKEND' in icheck:
                sms_service = icheck['SMS_BACKEND']
                if sms_service_check(sms_service):
                    # send sms:
                    match sms_service:
                        case 'PARSIAN_WEBCO_IR':
                            assert 'template_id' in request.data, 'template_id is required, in the PARSIAN_WEBCO_IR.'
                            template_id = request.data['template_id']
                            service = ParsianWebcoIr(mobile=mobile_number)
                            response = service.send_message(message=message_text, template_id=template_id)
                            if isinstance(response, dict) and 'status' in response:
                                if response['status'] == 200:
                                    return Response({'receiver': mobile_number, 'message': message_text}, status=HTTP_200_OK)
                                elif response['status'] == 100:
                                    return Response({'details': 'The SMS service provider was unable to process the request.'}, status=HTTP_502_BAD_GATEWAY)
                                elif response['status'] == 401:
                                    return Response({'details': 'Authentication is not accepted, check your token and if the token has not been received, obtain it from your SMS service provider.'}, status=HTTP_401_UNAUTHORIZED)
                        case 'MELI_PAYAMAK_COM':
                                service = MeliPayamakCom(mobile=mobile_number)
                                response = service.send_message(message=message_text)
                                if response in [0, 2, 3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, 16, 17, 35]:
                                    return Response({'details': 'The SMS service provider was unable to process the request.', 'errorcode': response}, status=HTTP_502_BAD_GATEWAY)
                                else:
                                    return Response({'receiver': mobile_number, 'message': message_text, 'messageid': response}, status=HTTP_200_OK)
                        case 'KAVENEGAR_COM':
                            service = KavenegarCom(mobile=mobile_number)
                            response = service.send_message(message=message_text)
                            try:
                                response_json = response.json()
                                entries = response_json.get('entries', [])
                                _return = response_json.get('return', [])
                                if entries:
                                    response_data = entries[0]
                                    if response.status_code == 200 and response_data.get('status') in [5, 10]:
                                        return Response({'receiver': response_data.get('receptor', ''), 'message': response_data.get('message', ''), 'messageid': response_data.get('messageid', '')}, status=HTTP_200_OK)
                                    else:
                                        return Response({'details': 'The SMS service provider was unable to process the request.', 'errorcode': response_data.get('status', ''), 'errortext': response_data.get('statustext', ''), 'message': response_data.get('message', '')}, HTTP_502_BAD_GATEWAY)
                                elif  _return:
                                    return Response({'details': 'The SMS service provider was unable to process the request.', 'message': _return.get('message', '')}, status=HTTP_502_BAD_GATEWAY)
                            except (ValueError, KeyError, IndexError) as e:
                                return Response({'details': 'Invalid response structure.', 'error': str(e)}, status=HTTP_502_BAD_GATEWAY)

        except AssertionError as e:
            return Response({'error': str(e)}, status=HTTP_204_NO_CONTENT)
        except:
            pass
        return Response({'error': 'An error occurred while generating or sending the otpcode, please contact the www.chelseru.com support team.'}, status=HTTP_500_INTERNAL_SERVER_ERROR)



from rest_framework.serializers import ModelSerializer
from django.contrib.auth.models import User
from .models import User as mobile, OTPCode


class DefaultUserSerializer(ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class MobileSerializer(ModelSerializer):
    class Meta:
        model = mobile
        fields = '__all__'
        depth = 1

class OTPCodeSerializer(ModelSerializer):
    class Meta:
        model = OTPCode
        fields = '__all__'




from setuptools import setup, find_packages

setup(
    name='django-iran-sms',
    version='1.3.9',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django-chelseru',
    ],
    author='Sobhan Bahman Rashnu',
    author_email='bahmanrashnu@gmail.com',
    description='A Django package for seamless integration with Iranian SMS services like ParsianWebCo , Kavenegar and Melipayamak.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://pipdjango.chelseru.com/',
    project_urls={
        "Documentation": "https://github.com/Chelseru/django-chelseru-lour/",
        "Telegram Group": "https://t.me/bahmanpy",
        "Telegram Channel": "https://t.me/djangochelseru",
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.11',
    keywords="djangoiransms djangosms drfsms drfiransms chelseru lor lur bahman rashnu sobhan bahman bahman rashnu melipayamak parsianwebco sms جنگو پیامک ملی اایران کاوه نگار python kavenegar kave negar meli payamak",
)



from django.contrib.auth.models import User as DefaultUser
from .models import User
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver

# @receiver(post_save, sender=User)
# def create_user_profile(sender, instance, created, **kwargs):
#     if created:
#         mobile.objects.create(user=instance)

# @receiver(post_save, sender=User)
# def save_user_profile(sender, instance, **kwargs):
#     instance.mobile.save()

@receiver(pre_save, sender=User)
def create_user_if_not_exists(sender, instance, **kwargs):
    if not instance.user_id:
        default_user, created = DefaultUser.objects.get_or_create(mobile__mobile=instance.mobile, mobile__group=instance.group, 
                                                                  defaults={'username': f'G{instance.group}-{instance.mobile}'})
        if created:
            instance.user = default_user